package com.oqpo.api.service.sttlmng;


import com.oqpo.api.entity.settmng.StlMgntBaseInfoEntity;
import com.oqpo.api.enums.SttlStCd;
import com.oqpo.api.mapper.sttlmng.SttlReqMapper;
import com.oqpo.api.mapper.sttlmng.SttlStateMapper;
import com.oqpo.api.service.CommonService;
import com.oqpo.api.util.DateUtil;
import com.oqpo.api.web.dto.request.GridRequest;
import com.oqpo.api.web.dto.request.sttlmng.SttlConfirmRequest;
import com.oqpo.api.web.dto.response.GridResponse;
import com.oqpo.api.web.dto.response.sttlmng.SttlStateGridResponse;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.List;
import java.util.stream.Collectors;

@Service
@Slf4j
public class SttlStateService extends CommonService {

    @Autowired
    private SttlStateMapper sttlStateMapper;

    @Autowired
    private SttlReqMapper sttlReqMapper;

    /*
      정산요청 그리드 리스트 조회
     */
    public GridResponse selectSttlStateGridList(String sGubun, String sBrandId, String sStoreId, String sSpplyId, String sItemNm,
                                                String sSttlStCd, String fromDt, String toDt, GridRequest gridRequest) throws Exception {
        int gridPage = gridRequest.getGridPage();
        int gridSize = gridRequest.getGridSize();

        fromDt = fromDt == null ? null : fromDt.replace(".", "");
        toDt = toDt == null ? null : toDt.replace(".", "");

        int gridRecords = sttlStateMapper.selectSttlStateGridCnt(sGubun, sBrandId, sStoreId, sSpplyId, sItemNm, sSttlStCd, fromDt, toDt);
        int gridTotal = fnCalculateGridTotal(gridSize, gridRecords);
        List<StlMgntBaseInfoEntity> entities = sttlStateMapper.selectSttlStateGridList(sGubun, sBrandId, sStoreId, sSpplyId, sItemNm, sSttlStCd, fromDt, toDt, gridRequest);
        List<Object> gridRows = entities.stream()
                .map(m -> SttlStateGridResponse.builder()
                        .viewCd("R")
                        .sttlMgntUnqNo(m.getSttlMgntUnqNo())        /* 정산관리고유번호 */
                        .brandId(m.getBrandId())
                        .brandNm(m.getBrandNm())
                        .storeId(m.getStoreId())
                        .storeNm(m.getStoreNm())
                        .spplyId(m.getSpplyNm())
                        .spplyNm(m.getSpplyNm())
                        .sttlStCd(m.getSttlStCd())                  /* 정산상태코드 */
                        .sttlStNm(m.getSttlStNm())                  /* 정산상태명 */
                        .sttlReqDt(m.getSttlReqDt())
                        .sttlReqAmt(m.getSttlReqAmt())
                        .sttlDt(m.getSttlDt())
                        .sttlAmt(m.getSttlAmt())
                        .acctBal(m.getAcctBal())
                        .sttlRegMgrId(m.getSttlRegMgrId())
                        .sttlRegMgrNm(m.getSttlRegMgrNm())
                        .build())
                .collect(Collectors.toList());
        return GridResponse.toDTO(gridPage, gridTotal, gridRecords, gridRows);
    }

    /* 정산완료  */
    @Transactional
    public void endSttl(String userId, SttlConfirmRequest sttlConfirmRequest) throws Exception {
        // 정산기본정보 조회
        StlMgntBaseInfoEntity entity = sttlReqMapper.selectStlMgntBaseInfo(sttlConfirmRequest.getSttlMgntUnqNo());

        // 정산관리기본정보 - 정산상태코드 변경
        StlMgntBaseInfoEntity infoEntity = new StlMgntBaseInfoEntity();
        infoEntity.setSttlMgntUnqNo(sttlConfirmRequest.getSttlMgntUnqNo());
        infoEntity.setSttlStCd(SttlStCd.STTL_CONFIRM.getCd());
        infoEntity.setSttlDt(DateUtil.getCurrentDate());
        infoEntity.setSttlAmt(entity.getSttlReqAmt()); // 정산요청금액을 정산금액으로 지정한다.
        sttlReqMapper.updateStlMgntBaseInfo4SttlConfirm(userId, infoEntity);
    }

}
